var app = app || {};

app.page = Backbone.Model.extend({
	defaults:{
		local:false,
		playing:false,
		coming:false,
	}
});

app.item = Backbone.Model.extend({
	defaults:{
		title:"",
		score:null,
		type:"",
		date:"",
		time:"",
		leader:""
	}
});

app.ref = {
	ref:0,

	inc:function(){
		this.ref++;
		this.refresh();
	},

	dec:function(){
		this.ref--;
		this.refresh();
	},

	refresh:function(){
		if (this.ref > 0)
			$("#mainLoader").removeClass('hidden');
		else
			$("#mainLoader").addClass('hidden');
	}
};

app.units = {
	popItemTemplate:_.template($("#popItem-template").html()),

	parseFilmsInfo:function(item,context){
		var type;
		if (_.isString(item['type']))
			type = item['type'].split(',');
		else
			type = item['type'];

		var typeString = '';
		for (var i in type)
			typeString += "<a href='#' class='tag'>" + type[i] + "</a>,";

		typeString = typeString.substring(0,typeString.length - 1);

		var leader;
		if (_.isString(item['leader']))
			leader = item['leader'].split(',');
		else
			leader = item['leader'];

		var leaderString = '';
		for (var i in leader)
			leaderString += "<a href='#' class='tag'>" + leader[i] + "</a>,";

		leaderString = leaderString.substring(0,leaderString.length - 1);

		var name = app.units.popItemTemplate(item);

		if (context)
		{
			try{
				app.db.updateInfo(item['name'],String(context),String(item['type']),item['date'],item['time'],item['score'],item['doubanID'],item['cover'],String(item['leader']),item['summary']);
			}catch(e){
				//影片名称重复，一般可能在剧集中出现，标识失败，人工处理。
				app.db.custom().prepare("update films set state=2 where filePath=?").exec(String(context));
			}
		}

		name = $(name).attr('filePath',context).prop("outerHTML");

		if (!_.isUndefined(item['filePath']))
			name = $(name).attr('filePath',item['filePath']).prop("outerHTML");

		return [name,item['score'],typeString,item['date'],item['time'],leaderString];
	},

	addFilmsCallback:function(item,context){

		var result = app.units.parseFilmsInfo(item,context);

		app.units.addMultilFilms(result);
	},

	addMultilFilms:function(items){
		$("#filmsInfo").dataTable().fnAddData(items);
		$("a").popover({html:true,trigger:"hover"});
	},

	getFilmsName:function(fileName){

		var matched = fileName.match(/.*\\(.*)\.(S[\d]+)?\.(.*\.)?([\d]{4})/i);
		if (matched)
		{
			return matched[1] + "." + matched[4] + (matched[2] ? "." + matched[2] : '');
		}
		else
		{
			matched = fileName.match(/.*\\(.*)\.([\d]{4})\.(S[\d]+)?/i);
			if (!matched)
				return;

			return matched[1] + "." + matched[2] + (matched[3] ? "." + matched[3] : '');
		}
	},

	failedTips:function(){
		$.jGrowl("获取数据失败，有可能被豆瓣识别成了机器人，请输入验证码！<a href='#' onclick=\"createWindow({url:'http://movie.douban.com',inherit:true}).show(true);\">打开豆瓣页面</a>", {
			beforeOpen: function(e,m,o){
		          $(e).width( "450px" );
		      },life: 1000 * 3,position:"bottom-left"
		});
	}
}

app.mainView = Backbone.View.extend({
    el:"#content",

    events:{
      'keyup #in_filter':'onFilter',
      'change #in_filter':'onFilter',
      'click #btn_reset':"onReset",

      'click #link_local':'onLocal',
      'click #link_playing':'onPlaying',
      'click #link_coming':'onComing',
      'click #link_refresh':'onRefresh',
      'click #link_setting':'onSetting',
      'click #link_add':'onAdd',
      'click #setting_browser':"onSettingBrowser",
      'click #setting_ok':"onSettingOK",

      'click .link_name':"onLinkName",
      'click .tag':"onTag",

      'click #add_browser':"onAddBrowser",
      "click #add_ok":"onAddOK",
      "click #add_remove":"onAddRemove",

      'click #link_feedback':"onFeedback",

      'contextmenu tr':"onContextMenu"
    },

    navTemplate:_.template($("#nav-template").html()),
    filmsTemplate:_.template($("#films-template").html()),
    footerTemplate:_.template($("#footer-template").html()),
    addTemplate:_.template($("#add-template").html()),
    settingTemplate:_.template($("#setting-template").html()),

    initialize:function(){
    	this.renderNav((new app.page({local:true}).toJSON())).renderFilms().renderFooter();

    	$("body").resize(_.bind(function(){
    		$("#filmsInfo").dataTable().fnSettings().oScroll.sY = win.height - 300;
    		$("#filmsInfo").dataTable().fnDraw();
		},this));

		app.db.initialize();
		this.onLocal();
    },

	renderNav:function(param){
		$("#top").html(this.navTemplate(param));
		app.ref.refresh();
		return this;
	},

	renderFilms:function(){

		if ($('#filmsInfo').length > 0)
		{
			$('#filmsInfo').dataTable().fnClearTable(0);
			$('#filmsInfo').dataTable().fnDraw(0);
		}
		else
		{
			$("#middle").html(this.filmsTemplate());

			$('#filmsInfo').dataTable({
				"sDom": 'rt<"bottom"lpi><"clear">',
				"paging":   false,
				"info":     true,
				"scrollY":  (win.height - 300) + "px",
				"scrollCollapse": true,
			});
		}

		return this;
	},

	renderFooter:function(){
		$("#bottom").html(this.footerTemplate());
		return this;
	},

	onReset:function(){
		$("#in_filter").val('').trigger('change');
	},

	onFilter:function(e){
		$('#filmsInfo').DataTable().search($(e.currentTarget).val()).draw();
	},

	onLocal:function(){
		this.renderNav((new app.page({local:true}).toJSON())).renderFilms();
			
		app.films.getLocal(app.units.addFilmsCallback);

		if (!app.db.querySet('path'))
			this.onSetting();
	},

	onPlaying:function(){
		this.renderNav((new app.page({playing:true}).toJSON())).renderFilms();
		app.films.getPlaying(app.units.addFilmsCallback);
	},

	onComing:function(){
		this.renderNav((new app.page({coming:true}).toJSON())).renderFilms();
		app.films.getComing(app.units.addFilmsCallback);
	},

	onAdd:function(){
		$("#popModal .modal-body").html(this.addTemplate({title:'添加',method:'add',doubanID:null,filePath:null}));
		$("#popModal").modal('show');
	},

	onRefresh:function(){
		$(".nav .active > a").trigger('click');
	},

	onSetting:function(){
		$("#popModal .modal-body").html(this.settingTemplate({path:app.db.querySet('path'),autoDel:app.db.querySet('autoDel')}))
		$("#popModal").modal('show');
	},

	onSettingBrowser:function(){
		var dir = win.selectFolder({});
		if (dir)
			$("#setting_path").val(dir);
	},

	onSettingOK:function(){
		app.db.saveSet({path:$("#setting_path").val()});
		app.db.saveSet({autoDel:$("#chk_autoDel").prop('checked')});

		this.onRefresh();
	},

	onFeedback:function(){
		if (_.isUndefined(this.feedback))
			this.feedback = new app.feedback();
		else
			this.feedback.initialize();
	},

	onLinkName:function(e){
		if (window.event.ctrlKey)
		{
			e.preventDefault();

			var filePath = $(e.currentTarget).attr('filePath');

			filePath = filePath.split("|");

			if (filePath.length > 1)
				sys.shellExecute({path:sys.expandEnvirPath("%systemroot%\\explorer.exe"),args:"/select," + "\"" + filePath[0] + "\\..\\\""});
			else
				sys.shellExecute({path:sys.expandEnvirPath("%systemroot%\\explorer.exe"),args:"/select," + "\"" + filePath[0] + "\""});

			return;
		}
		else
		{
			var doubanID = $(e.currentTarget).attr('doubanID');
			var name = $(e.currentTarget).text();

			if (doubanID)
				sys.shellExecute({path:"http://movie.douban.com/subject/" + doubanID + "/"});
			else
				sys.shellExecute({path:"http://movie.douban.com/subject_search?search_text=" + name.replace(/\./g,'+')});
		}
	},

	onTag:function(e){
		if (window.event.ctrlKey)
		{
		}
		else
		{
			$("#in_filter").val($("#in_filter").val() + " " + $(e.currentTarget).text()).trigger('change');
		}
	},

	onAddBrowser:function(e){
		var result = win.selectFile({fileMustExist:true,filter:"*.mkv\0*.mkv"});

		if (result)
			$("#filePath").val(result);
	},

	onAddOK:function(e){
		var subject = $("#link_douban").val().match(/http:\/\/movie\.douban\.com\/subject\/([\d]+)[\/]?/);

		if (!subject)
		{
			alert('不识别豆瓣链接！')
			return;
		}

		if ($("#filePath").val() == '')
		{
			alert('无效的文件路径');
			return;
		}

		if (subject)
		{
			var name = app.units.getFilmsName($("#filePath").val());

			if ($("#popModal").find("data").attr("method") == 'add')
			{
				var result = app.db.custom().prepare("select * from films where filePath=?").exec($("#filePath").val());
				if (result.length > 0)
				{
					alert('记录已存在！');
					return;
				}

				try{
					app.db.custom().prepare("insert into films(name,filePath,doubanID) values(?,?,?)").exec(name,$("#filePath").val(),subject[1]);	
				}catch(e){
					alert('插入新数据异常，可能是电影名称重复！');
					return;
				}
			}
			else if ($("#popModal").find("data").attr("method") == 'edit')
			{
				app.db.custom().prepare("update films set state=0,doubanID=?,filePath=? where filePath=?").exec(subject[1],$("#filePath").val(),$("#popModal").find("data").attr("filePath"));
			}

			ga.event('revise',name + " => " + subject[1]);
		}

		this.onRefresh();
		$('#popModal').modal('hide');		
	},

	onAddRemove:function(e){
		app.db.custom().prepare("update films set state=3 where filePath=?").exec($("#addModal").find("data").attr("filePath"));

		this.onRefresh();
	},

	onContextMenu:function(e){
		var item = $(e.currentTarget).find('td:first a');

		$("#popModal .modal-body").html(this.addTemplate({title:'编辑',method:'edit',doubanID:item.attr('doubanID'),filePath:item.attr('filePath')}));
		$("#popModal").modal('show');

		
		//alert($(e.currentTarget).find('td:first a').attr('filePath'));
	}
})